"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.config import Config
from seerbit.interface.app_interface import IClientConstants
from seerbit.enums import EnvironmentEnum, AuthTypeEnum
from seerbit.exception import SeerBitError


class Client:

    def __init__(self, version=None):
        self.config = Config()
        self.config.put("authentication_scheme", str(AuthTypeEnum.BEARER.value))
        if version is None:
            self.config.put("version", IClientConstants.VERSION_TWO)
        else:
            if "1.0.1" == version:
                self.config.put("version", IClientConstants.VERSION_TWO)
            else:
                raise SeerBitError("Version must be \"1.0.1\"")

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, config: Config):
        self._config = config

    @property
    def public_key(self):
        return self._config.get("public_key")

    @public_key.setter
    def public_key(self, public_key: str):
        self._config.put("public_key", public_key)

    @property
    def private_key(self):
        return self._config.get("private_key")

    @private_key.setter
    def private_key(self, public_key: str):
        self._config.put("private_key", public_key)

    @property
    def version(self):
        return str(self._config.get("version"))

    @version.setter
    def version(self, version: str):
        if "1.0.1" == version:
            self._config.put("version", IClientConstants.VERSION_TWO)
        else:
            raise SeerBitError("Version must be \"1.0.1\"")

    @property
    def environment(self):
        env = ""
        if self.config.get("environment") is not None:
            env = str(self.config.get("environment"))
        return env

    @environment.setter
    def environment(self, environment: str):
        if environment == str(EnvironmentEnum.LIVE.value):
            self._config.put("environment", str(EnvironmentEnum.LIVE.value))
            self._config.put("api_base", IClientConstants.LIVE_API_BASE)
        elif environment == str(EnvironmentEnum.TEST.value):
            self._config.put("environment", str(EnvironmentEnum.TEST.value))
            self._config.put("api_base", IClientConstants.TEST_API_BASE)
        else:
            msg = "This environment does not exist, use \"{0}\" or \"{1}\""
            error_message = msg.format(EnvironmentEnum.LIVE.value, EnvironmentEnum.TEST.value)
            raise SeerBitError(error_message)

    @property
    def authentication_scheme(self):
        auth_type: str = ""
        if self.config.get("authentication_scheme") is not None:
            auth_type = str(self.config.get("authentication_scheme"))
        return auth_type

    @authentication_scheme.setter
    def authentication_scheme(self, auth_type: str):
        auth_type: str = auth_type.lower()
        if auth_type in ["basic ", "bearer "]:
            self.config.put("authentication_scheme", auth_type)
        else:
            raise SeerBitError("Invalid Authentication Scheme")

    @property
    def api_base(self):
        return str(self._config.get("api_base"))

    @api_base.setter
    def api_base(self, api_base: str):
        self._config.put("api_base", api_base)

    @property
    def timeout(self):
        return self._config.get("timeout")

    @timeout.setter
    def timeout(self, timeout):
        self._config.put("timeout", timeout)



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.exception import SeerBitError
from seerbit.interface.app_interface import IClientConstants
from seerbit.interface.service_interface import IStandardCheckoutService
from seerbit.service.servicelib import Service
from seerbit.utility import Utility
from seerbit.validation import StandardCheckoutValidator


class StandardCheckoutService(Service, IStandardCheckoutService, IClientConstants):

    def __init__(self, client: Client, token: str):
        """

        :param Client client:
            A non optional Client, the client with config

        :param str token:
            A non optional string, the auth token

        """
        super(StandardCheckoutService, self).__init__(client)
        self.token = token
        Utility.non_null(client)

    def initialize_transaction(self, standard_checkout: dict):
        """

        POST /api/v2/payments

        :param dict standard_checkout:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        StandardCheckoutValidator.is_valid_checkout(schema=standard_checkout)
        endpoint = IClientConstants.INITIALIZE_TRANSACTIONS
        self.response = self.post_request(endpoint, standard_checkout, self.token)
        return self.response

    def get_hash(self, standard_checkout: dict) -> str:
        """

        POST /api/v2/encrypt/hashs

        :param dict standard_checkout:
            A non optional dict, the payload

        :returns str hash_str

        """
        self.requires_token = True
        StandardCheckoutValidator.is_valid_checkout(schema=standard_checkout)
        response = self.post_request(IClientConstants.HASH_REQUEST, standard_checkout, self.token)
        if response.get("data"):
            response = response.get("data")
            if response.get("hash"):
                hash_str = response["hash"]["hash"]
            else:
                raise SeerBitError("Unable to obtain hash")
        else:
            raise SeerBitError("Unable to obtain hash")
        return hash_str



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.interface.app_interface import IHttpClient, INumericConstants
from seerbit.enums import HttpHeaderEnum
from seerbit.exception import SeerBitConnectionError, SeerBitError
from requests import post, get, put


class HttpClient(IHttpClient, INumericConstants):

    def __init__(self):
        self.status_code = -20

    def post(self, service, request_url, params, token):
        client = service.client
        if service.requires_token:
            if not token or len(token) < self.MIN_SIZE:
                raise SeerBitConnectionError("Please provide an authentication token.")
            else:
                authentication_scheme = client.authentication_scheme
                if authentication_scheme.lower() == "basic ":
                    token = "Basic {0}".format(token)
                elif authentication_scheme.lower() == "bearer ":
                    token = "Bearer {0}".format(token)
                else:
                    raise SeerBitError("Invalid Authentication Scheme")
                header = {
                    "Authorization": token,
                    "Request-Timeout": str(service.client.timeout),
                    str(HttpHeaderEnum.CONTENT_TYPE_PARAM.value): str(HttpHeaderEnum.CONTENT_TYPE_VALUE.value)
                }
                response = post(url=request_url, json=params, headers=header)
        else:
            header = {
                "Request-Timeout": str(service.client.timeout),
                str(HttpHeaderEnum.CONTENT_TYPE_PARAM.value): str(HttpHeaderEnum.CONTENT_TYPE_VALUE.value)
            }
            response = post(url=request_url, json=params, headers=header)
        status_code = int(response.status_code)
        if status_code < self.HTTP_STATUS_200 or status_code > self.HTTP_STATUS_299:
            SeerBitError.handle_error(response)
        return response

    def put(self, service, request_url, params, token):
        client = service.client
        if service.requires_token:
            if not token or len(token) < self.MIN_SIZE:
                raise SeerBitConnectionError("Please provide an authentication token.")
            else:
                authentication_scheme = client.authentication_scheme
                if authentication_scheme.lower() == "basic ":
                    token = "Basic {0}".format(token)
                elif authentication_scheme.lower() == "bearer ":
                    token = "Bearer {0}".format(token)
                else:
                    raise SeerBitError("Invalid Authentication Scheme")
                header = {
                    "Authorization": token,
                    "Request-Timeout": str(service.client.timeout),
                    str(HttpHeaderEnum.CONTENT_TYPE_PARAM): str(HttpHeaderEnum.CONTENT_TYPE_VALUE)
                }
                response = put(url=request_url, json=params, headers=header)
        else:
            header = {
                str(HttpHeaderEnum.CONTENT_TYPE_PARAM.value): str(HttpHeaderEnum.CONTENT_TYPE_VALUE.value),
                "Request-Timeout": str(service.client.timeout)
            }
            response = put(url=request_url, json=params, headers=header)
        status_code = int(response.status_code)
        if status_code < self.HTTP_STATUS_200 or status_code > self.HTTP_STATUS_299:
            SeerBitError.handle_error(response)
        return response

    def get(self, service, request_url, token):
        client = service.client
        if service.requires_token:
            if not token or len(token) < self.MIN_SIZE:
                raise SeerBitConnectionError("Please provide an authentication token.")
            else:
                authentication_scheme = client.authentication_scheme
                if authentication_scheme.lower() == "basic ":
                    token = "Basic {0}".format(token)
                elif authentication_scheme.lower() == "bearer ":
                    token = "Bearer {0}".format(token)
                else:
                    raise SeerBitError("Invalid Authentication Scheme")
                header = {
                    "Authorization": token,
                    "Request-Timeout": str(service.client.timeout),
                    str(HttpHeaderEnum.CONTENT_TYPE_PARAM.value): str(HttpHeaderEnum.CONTENT_TYPE_VALUE.value)
                }
                response = get(url=request_url, headers=header)
        else:
            header = {
                str(HttpHeaderEnum.CONTENT_TYPE_PARAM.value): str(HttpHeaderEnum.CONTENT_TYPE_VALUE.value),
                "Request-Timeout": str(service.client.timeout)
            }
            response = get(url=request_url, headers=header)
        status_code = int(response.status_code)
        if status_code < self.HTTP_STATUS_200 or status_code > self.HTTP_STATUS_299:
            SeerBitError.handle_error(response)
        return response



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from base64 import b64encode
from seerbit.exception import SeerBitError
from seerbit.interface.app_interface import IClientConstants, INumericConstants
from seerbit.client import Client
from seerbit.config import Config
from seerbit.interface.service_interface import IAuthentication
from seerbit.service.servicelib import Service
from seerbit.utility import Utility


class Authentication(IAuthentication, Service, IClientConstants, INumericConstants):

    def __init__(self, client: Client):
        """

        :param Client client:
            A non optional Client, the client with config

        """
        super(Authentication, self).__init__(client)
        Utility.non_null(client)

    def auth(self):
        """

        POST /api/v2/encrypt/keys

        :returns Any self.response

        """
        config: Config = self.client.config
        payload: dict = {
            "key": None
        }
        key = config.private_key + "." + config.public_key
        payload["key"] = key
        self.response = self.post_request(IClientConstants.AUTHENTICATION_ENDPOINT, payload, None)
        return self.response

    def get_token(self) -> str:
        """

        :return str encrypted_key

        """
        encrypted_key: str = ""
        if self.response:
            if "data" in self.response:
                encrypted_key_dict = self.response["data"]
                if "EncryptedSecKey" in encrypted_key_dict:
                    encrypted_key_dict = encrypted_key_dict["EncryptedSecKey"]
                    encrypted_key = encrypted_key_dict.get("encryptedKey")
        return encrypted_key

    def get_basic_auth_encoded_string(self) -> str:
        """

        :return str base 64 encoded string

        """
        client = self.client
        authentication_scheme = client.authentication_scheme
        if authentication_scheme.lower() not in ["basic ", "bearer "]:
            raise SeerBitError("Set authentication scheme to AuthTypeEnum.BASIC value before calling this method")
        authorization_str = client.public_key + ":" + client.private_key
        encoded_bytes = b64encode(authorization_str.encode("utf-8"))
        encoded_str = str(encoded_bytes, "utf-8")
        return encoded_str



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from requests import Response


class SeerBitError(RuntimeError):

    def __init__(self, message=""):
        self.message = message
        super(SeerBitError, self).__init__(message)

    @staticmethod
    def handle_error(response: Response):
        json_object = response.json()
        error_message = "{0}: {1}"
        if json_object.get("status") and json_object.get("data"):
            status = json_object["status"]
            if status.lower() == "error":
                raise SeerBitError(json_object["data"]["message"])
        if json_object.get("message") and json_object.get("errorCode"):
            error_message = error_message.format(str(json_object["message"]), str(json_object["errorCode"]))
            print(error_message)
            raise SeerBitError(str(json_object["message"]))
        elif json_object["message"]:
            raise SeerBitError(json_object["message"])
        else:
            raise SeerBitError("Unknown Error")


class SeerBitConnectionError(RuntimeError):

    def __init__(self, message="", code=0, status=None, timestamp=None):
        self.message = message
        self.code = code
        self.status = status
        self.timestamp = timestamp
        super(SeerBitConnectionError, self).__init__(message, code, status, timestamp)



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """


# request
class IRequest(object):

    def post_request(self, endpoint, payload, token):
        pass

    def put_request(self, endpoint, payload, token):
        pass

    def get_request(self, endpoint, token):
        pass


class IUssdService(object):

    def payment_ussd(self, payload):
        pass


class ITransferService(object):

    def payment_transfer(self, payload):
        pass


class IService(object):

    @property
    def client(self):
        raise NotImplementedError

    @property
    def requires_token(self):
        raise NotImplementedError

    @requires_token.setter
    def requires_token(self, is_required):
        pass


# account service
class IAccountService(object):

    def authorize(self, payload):
        pass

    def validate(self, payload):
        pass


# auth service
class IAuthentication(object):

    def auth(self):
        pass

    def get_token(self) -> str:
        pass


# card service
class ICardService(object):

    def authorize(self, card: dict):
        pass

    def validate(self, transaction: dict):
        pass

    def preauth_authorization(self, card_pre_auth: dict):
        pass

    def payment_capture(self, payment_capture: dict):
        pass

    def payment_refund(self, payment_refund: dict):
        pass

    def payment_cancel(self, payment_cancel: dict):
        pass

    def payment_charge_non3d(self, payment_charge: dict):
        pass

    def payment_charge_3ds(self, payment_charge: dict):
        pass

    def payment_charge_3d(self, payment_charge: dict):
        pass

    def tokenize(self, payment_charge: dict):
        pass


# mobile money service
class IMobileMoneyService(object):

    def authorize(self, mobile_money):
        pass

    def get_available_networks(self):
        pass


# order service
class IOrderService(object):

    def authorize(self, card):
        pass


# resource service
class IResourceService(object):

    def get_bank_list(self, public_key):
        pass


# status service
class IStatusService(object):

    def get_transaction_status(self, payment_reference):
        pass

    def get_subscription_status(self, billing_id):
        pass


# recurrent service
class IRecurringService(object):

    def create_subscription(self, subscription):
        pass

    def get_customer_subscriptions(self, public_key, customer_id):
        pass

    def update_subscription(self, subscription: dict):
        pass

    def get_merchant_subscriptions(self, public_key):
        pass

    def recurring_debit(self, recurring_debit: dict):
        pass


# refund service
class IRefundService(object):

    def all_refunds(self, business_id, page, size):
        pass

    def get_refund(self, business_id, refund_id):
        pass

    def refund(self, business_id, payload):
        pass


# standard checkout service
class IStandardCheckoutService(object):

    def initialize_transaction(self, standard_checkout: dict):
        pass

    def get_hash(self, standard_checkout: dict) -> str:
        pass



"""
  Copyright (C) 2022 SeerBit
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.exception import SeerBitError


class Utility:

    @staticmethod
    def non_null(client: Client):
        if not client:
            raise SeerBitError("Client cannot be null")

    @staticmethod
    def require_non_null(api_base, error_message):
        if not api_base:
            raise SeerBitError(error_message)



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.interface.app_interface import IClientConstants
from seerbit.interface.service_interface import IAccountService
from seerbit.service.servicelib import Service
from seerbit.validation import AccountValidator
from seerbit.utility import Utility


class AccountService(Service, IAccountService, IClientConstants):

    def __init__(self, client: Client, token: str):
        """

        :param Client client:
            A non optional Client, the client with config

        :param str token:
            A non optional string, the auth token

        """
        super(AccountService, self).__init__(client)
        self.token = token
        Utility.non_null(client)

    def authorize(self, account_payload: dict):
        """

        POST /api/v2/payments/initiates

        :param dict account_payload:
            A non optional dict, the payload

        :returns: Any self.response

        """
        AccountValidator.is_valid_authorize(schema=account_payload)
        self.requires_token = True
        self.response = self.post_request(IClientConstants.INITIATE_PAYMENT_ENDPOINT, account_payload, self.token)
        return self.response

    def validate(self, otp_payload: dict):
        """

        POST /api/v2/payments/validate

        :param dict otp_payload:
            A non optional dict, the payload

        :returns: Any self.response

        """
        AccountValidator.is_valid_validate(schema=otp_payload)
        self.requires_token = True
        self.response = self.post_request(IClientConstants.VALIDATE_PAYMENT_ENDPOINT, otp_payload, self.token)
        return self.response



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.interface.app_interface import IConfig, INumericConstants


class Config(IConfig, INumericConstants):

    @property
    def timeout(self):
        return self.timeout

    def get(self, key: str):
        return Config.data[key] if Config.data is not None else None

    @property
    def public_key(self) -> str:
        public_key = ""
        if len(Config.data) != 0:
            public_key = Config.data.get("public_key")
        return public_key

    @property
    def private_key(self) -> str:
        private_key = ""
        if len(Config.data) != 0:
            private_key = Config.data.get("private_key")
        return private_key

    def put(self, key, value):
        Config.data[key] = value




"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.enums import EnvironmentEnum
from seerbit.exception import SeerBitError
from seerbit.httpclient import HttpClient
from seerbit.interface.service_interface import IService, IRequest
from seerbit.utility import Utility


class Service(IService, IRequest):
    _client: Client
    http_client: HttpClient
    message: str
    token: str
    response: dict

    def __init__(self, client: Client):
        """

        :param Client client:
            A non optional Client, the client with config

        """
        self._requires_token = False
        Utility.non_null(client)
        if not client.config.get("environment"):
            msg = "Client does not have correct environment. Use {0} or {1}"
            self.message = msg.format(EnvironmentEnum.LIVE.value, EnvironmentEnum.TEST.value)
            raise SeerBitError(self.message)
        if not client.config.get("public_key"):
            self.message = "Client doesn\'t have a merchant public key. Set a public key using the client"
            raise SeerBitError(self.message)
        if not client.config.get("private_key"):
            self.message = "Client doesn\'t have a merchant private key. Set a private key using the client"
            raise SeerBitError(self.message)
        self.http_client = HttpClient()
        Service._client = client

    @property
    def requires_token(self) -> bool:
        return self._requires_token

    @requires_token.setter
    def requires_token(self, is_required: bool):
        self._requires_token = is_required

    @property
    def client(self) -> Client:
        return Service._client

    @client.setter
    def client(self, client: Client):
        Utility.non_null(client)
        Service._client = client

    def post_request(self, endpoint, payload, token):
        message = "Set a field named \"api_base\" in the client configuration"
        Utility.require_non_null(self.client.config.get("api_base"), message)
        endpoint_url = str(self.client.config.get("api_base")) + endpoint
        json = self.http_client.post(self, endpoint_url, payload, token)
        return json.json()

    def put_request(self, endpoint, payload, token):
        message = "Set a field named \"api_base\" in the client configuration"
        Utility.require_non_null(self.client.config.get("api_base"), message)
        endpoint_url = str(self.client.config.get("api_base")) + endpoint
        json = self.http_client.put(self, endpoint_url, payload, token)
        return json.json()

    def get_request(self, endpoint, token):
        message = "Set a field named \"api_base\" in the client configuration"
        Utility.require_non_null(self.client.config.get("api_base"), message)
        endpoint_url = str(self.client.config.get("api_base")) + endpoint
        json = self.http_client.get(self, endpoint_url, token)
        return json.json()



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.interface.app_interface import IClientConstants
from seerbit.interface.service_interface import IRecurringService
from seerbit.service.servicelib import Service
from seerbit.utility import Utility
from seerbit.validation import RecurringValidator


class RecurringService(Service, IRecurringService, IClientConstants):

    def __init__(self, client: Client, token: str):
        """

        :param Client client:
            A non optional Client, the client with config

        :param str token:
            A non optional string, the auth token

        """
        super(RecurringService, self).__init__(client)
        self.token = token
        Utility.non_null(client)

    def create_subscription(self, subscription: dict):
        """

        POST /api/v2/recurring/subscribes

        :param dict subscription:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        RecurringValidator.is_valid_create_subscription(schema=subscription)
        self.response = self.post_request(IClientConstants.SUBSCRIPTION_ENDPOINT, subscription, self.token)
        return self.response

    def get_customer_subscriptions(self, public_key: str, customer_id: str):
        """

        GET /api/v2/recurring/{public_key}/customerId/{customer_id}

        :param str public_key:
            A non optional string, the merchant public key

        :param str customer_id:
            A non optional string, the customer id

        :returns Any self.response

        """
        self.requires_token = True
        endpoint = IClientConstants.CUSTOMER_SUBSCRIPTION_ENDPOINT.format(public_key, customer_id)
        self.response = self.get_request(endpoint, self.token)
        return self.response

    def update_subscription(self, subscription: dict):
        """
        PUT /api/v2/recurring/updates

        :param dict subscription:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        RecurringValidator.is_valid_update_subscription(schema=subscription)
        self.response = self.put_request(IClientConstants.UPDATE_SUBSCRIPTION_ENDPOINT, subscription, self.token)
        return self.response

    def get_merchant_subscriptions(self, public_key: str):
        """

        GET /api/v2/recurring/publicKey/{public_key}

        :param str public_key:
            A non optional string, the merchant public key

        :returns Any self.response

        """
        self.requires_token = True
        endpoint = IClientConstants.MERCHANT_SUBSCRIPTIONS_ENDPOINT.format(public_key)
        self.response = self.get_request(endpoint, self.token)
        return self.response

    def recurring_debit(self, recurring_debit: dict):
        """

        POST /api/v2/recurring/charge

        :param dict recurring_debit:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        RecurringValidator.is_valid_recurring_debit(schema=recurring_debit)
        self.response = self.post_request(IClientConstants.CHARGE_ENDPOINT, recurring_debit, self.token)
        return self.response



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """


class TransferValidator(object):

    @staticmethod
    def is_valid_payment_transfer(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "mobileNumber" not in schema:
            msg += "\"mobileNumber\" field is required\n"
        if "callbackUrl" not in schema:
            msg += "\"callbackUrl\" field is required\n"
        if "redirectUrl" not in schema:
            msg += "\"redirectUrl\" field is required\n"
        if "fullName" not in schema:
            msg += "\"fullName\" field is required\n"
        if "paymentType" not in schema:
            msg += "\"paymentType\" field is required\n"
            if schema["paymentType"] != "TRANSFER":
                msg += "\"paymentType\" field must be set as \"TRANSFER\""
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class UssdValidator(object):

    @staticmethod
    def is_valid_payment_ussd(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "mobileNumber" not in schema:
            msg += "\"mobileNumber\" field is required\n"
        if "bankCode" not in schema:
            msg += "\"bankCode\" field is required\n"
        if "callbackUrl" not in schema:
            msg += "\"callbackUrl\" field is required\n"
        if "redirectUrl" not in schema:
            msg += "\"redirectUrl\" field is required\n"
        if "fullName" not in schema:
            msg += "\"fullName\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class AccountValidator(object):

    @staticmethod
    def is_valid_authorize(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "accountName" not in schema:
            msg += "\"accountName\" field is required\n"
        if "bankCode" not in schema:
            msg += "\"bankCode\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "paymentType" not in schema:
            msg += "\"paymentType\" field is required\n"
        if msg != "\"paymentType\" field is required\n":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_validate(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "otp" not in schema:
            msg += "\"otp\" field is required\n"
        if "linkingReference" not in schema:
            msg += "\"linkingReference\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class CardValidator(object):

    @staticmethod
    def is_valid_authorize(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "fullName" not in schema:
            msg += "\"fullName\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_validate(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "transaction" not in schema:
            msg += "\"transaction\" field is required\n"
        elif schema["transaction"]:
            schema = schema["transaction"]
            if "otp" not in schema:
                msg += "\"otp\" field is required\n"
            if "linkingreference" not in schema:
                msg += "\"linkingreference\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_preauth(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_capture(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "productDescription" not in schema:
            msg += "\"productDescription\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_refund(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_cancel(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_charge_non3d(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_charge_3ds(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "fullName" not in schema:
            msg += "\"fullName\" field is required\n"
        if "paymentType" not in schema:
            msg += "\"paymentType\" field is required\n"
        if "channelType" not in schema:
            msg += "\"channelType\" field is required\n"
        if "retry" not in schema:
            msg += "\"retry\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_payment_charge_3d(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "fullName" not in schema:
            msg += "\"fullName\" field is required\n"
        if "paymentType" not in schema:
            msg += "\"paymentType\" field is required\n"
        if "channelType" not in schema:
            msg += "\"channelType\" field is required\n"
        if "retry" not in schema:
            msg += "\"retry\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_tokenize(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class MobileMoneyValidator(object):

    @staticmethod
    def is_valid_authorize(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "mobileNumber" not in schema:
            msg += "\"mobileNumber\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "paymentType" not in schema:
            msg += "\"paymentType\" field is required\n"
        if "network" not in schema:
            msg += "\"network\" field is required\n"
        if schema["network"]:
            if schema["network"] not in ["AIRTEL", "TIG", "VODAFONE", "MTN"]:
                msg += "enter a valid network provider\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class OrderValidator(object):

    @staticmethod
    def is_valid_authorize(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "orderType" not in schema:
            msg += "\"orderType\" field is required\n"
        if "orders" in schema:
            for orders in schema["orders"]:
                if "orderId" not in orders:
                    msg += "\"orderId\" field in \"orders\" element is required\n"
                    break
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class RecurringValidator(object):

    @staticmethod
    def is_valid_create_subscription(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "cardNumber" not in schema:
            msg += "\"cardNumber\" field is required\n"
        if "expiryMonth" not in schema:
            msg += "\"expiryMonth\" field is required\n"
        if "callbackUrl" not in schema:
            msg += "\"callbackUrl\" field is required\n"
        if "expiryYear" not in schema:
            msg += "\"expiryYear\" field is required\n"
        if "cvv" not in schema:
            msg += "\"cvv\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "startDate" not in schema:
            msg += "\"startDate\" field is required\n"
        if "billingCycle" not in schema:
            msg += "\"billingCycle\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "billingPeriod" not in schema:
            msg += "\"billingPeriod\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_update_subscription(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "mobileNumber" not in schema:
            msg += "\"mobileNumber\" field is required\n"
        if "status" not in schema:
            msg += "\"status\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid

    @staticmethod
    def is_valid_recurring_debit(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "authorizationCode" not in schema:
            msg += "\"authorizationCode\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid


class StandardCheckoutValidator(object):

    @staticmethod
    def is_valid_checkout(schema: dict) -> bool:
        msg = ""
        is_valid = True
        if "publicKey" not in schema:
            msg += "\"publicKey\" field is required\n"
        if "paymentReference" not in schema:
            msg += "\"paymentReference\" field is required\n"
        if "callbackUrl" not in schema:
            msg += "\"callbackUrl\" field is required\n"
        if "amount" not in schema:
            msg += "\"amount\" field is required\n"
        if "currency" not in schema:
            msg += "\"currency\" field is required\n"
        if "country" not in schema:
            msg += "\"country\" field is required\n"
        if "email" not in schema:
            msg += "\"email\" field is required\n"
        if "productId" not in schema:
            msg += "\"productId\" field is required\n"
        if "productDescription" not in schema:
            msg += "\"productDescription\" field is required\n"
        if msg != "":
            is_valid = False
        if not is_valid:
            raise ValueError(msg)
        return is_valid



import setuptools

with open("README.md", "r") as file:
    long_description = file.read()

setuptools.setup(
    name="seerbit-python-v2",
    version="1.0.4",
    author="SeerBit",
    author_email="developers@seerbit.com",
    description="A SeerBit API Library for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/seerbit/seerbit-python-api-library",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.9',
)



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from enum import Enum


class EnvironmentEnum(Enum):
    TEST = "test"
    LIVE = "live"


class RefundType(Enum):
    FULL_REFUND = "FULL_REFUND"
    PARTIAL_REFUND = "PARTIAL_REFUND"


class HttpHeaderEnum(Enum):
    CONTENT_TYPE_PARAM = "Content-Type"
    CONTENT_TYPE_VALUE = "application/json"


class AuthTypeEnum(Enum):
    BASIC = "Basic "
    BEARER = "Bearer "



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.interface.app_interface import ISeerBit, IClientConstants


class SeerBit(ISeerBit, IClientConstants):

    @property
    def api_version(self):
        return self.api_version

    @api_version.setter
    def api_version(self, api_version):
        self.api_version = api_version

    @property
    def live_api_base(self):
        return self.LIVE_API_BASE

    @property
    def test_api_base(self):
        return self.TEST_API_BASE




"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """


class INumericConstants(object):
    # numeric constants
    MIN_VALUE = 1
    MIN_SIZE = 1
    NIL = 0
    HTTP_STATUS_200 = 200
    HTTP_STATUS_299 = 299


class ISeerBit(object):

    @property
    def api_version(self):
        raise NotImplementedError

    @api_version.setter
    def api_version(self, api_version):
        pass

    @property
    def live_api_base(self):
        raise NotImplementedError

    @property
    def test_api_base(self):
        raise NotImplementedError


class IConfig(object):
    data = {}

    def get(self, key):
        pass

    @property
    def public_key(self):
        raise NotImplementedError

    @property
    def private_key(self):
        raise NotImplementedError

    def put(self, key, value):
        pass

    @property
    def timeout(self):
        raise NotImplementedError


class IClientConstants(object):
    # base url
    TEST_API_BASE = "https://pilot-backend.seerbitapi.com/"
    LIVE_API_BASE = "https://seerbitapi.com/"

    # authentication
    AUTHENTICATION_ENDPOINT = "api/v2/encrypt/keys"
    HASH_REQUEST = "api/v2/encrypt/hashs"

    # bank & card payment options
    INITIALIZE_TRANSACTIONS = "api/v2/payments"
    INITIATE_PAYMENT_ENDPOINT = "api/v2/payments/initiates"
    VALIDATE_PAYMENT_ENDPOINT = "api/v2/payments/validate"
    VALIDATE_CARD_PAYMENT_ENDPOINT = "api/v2/payments/otp"
    PREAUTH_AUTHORIZE_ENDPOINT = "api/v2/payments/authorise"
    PAYMENT_CAPTURE_ENDPOINT = "api/v2/payments/capture"
    PAYMENT_REFUND_ENDPOINT = "api/v2/payments/refund"
    PAYMENT_CANCEL_ENDPOINT = "api/v2/payments/cancel"
    PAYMENT_CHARGE_ENDPOINT = "api/v2/payments/charge"
    TOKENIZATION_ENDPOINT = "api/v2/payments/tokenize"

    # refunds
    REFUND_ENDPOINT = "merchants/api/v1/user/{0}/refunds"
    REFUND_DETAIL_ENDPOINT = "merchants/api/v1/user/{0}/refunds/{1}"
    REFUND_LIST_ENDPOINT = "merchants/api/v1/user/{0}/refunds?page={1}&size={2}"

    # orders
    ORDERS_ENDPOINT = "api/v2/payments/order"

    # mobile money
    AVAILABLE_NETWORKS_ENDPOINT = "api/v2/networks"

    # recurring
    SUBSCRIPTION_ENDPOINT = "api/v2/recurring/subscribes"
    CUSTOMER_SUBSCRIPTION_ENDPOINT = "api/v2/recurring/{0}/customerId/{1}"
    UPDATE_SUBSCRIPTION_ENDPOINT = "api/v2/recurring/updates"
    MERCHANT_SUBSCRIPTIONS_ENDPOINT = "api/v2/recurring/publicKey/{0}"
    CHARGE_ENDPOINT = "api/v2/recurring/charge"

    # status queries
    TRX_STATUS_ENDPOINT = "api/v2/payments/query/{0}"
    SUBSCRIPTION_STATUS_ENDPOINT = "api/v2/recurring/billingId/{0}"

    # bank list
    BANK_LIST_ENDPOINT = "api/v2/banks/merchant/{0}"

    # version types
    VERSION_ONE = "1.0.0"
    VERSION_TWO = "1.0.1"


class IHttpClient(object):

    def post(self, service, request_url, params, token):
        pass

    def put(self, service, request_url, params, token):
        pass

    def get(self, service, request_url, token):
        pass



"""
  Copyright (C) 2022 SeerBit

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
from seerbit.client import Client
from seerbit.interface.app_interface import IClientConstants
from seerbit.interface.service_interface import ICardService
from seerbit.service.servicelib import Service
from seerbit.validation import CardValidator
from seerbit.utility import Utility


class CardService(Service, ICardService, IClientConstants):

    def __init__(self, client: Client, token: str):
        """

        :param Client client:
            A non optional Client, the client with config

        :param str token:
            A non optional string, the auth token

        """
        super().__init__(client)
        self.token = token
        Utility.non_null(client)

    def authorize(self, card: dict):
        """

        POST /api/v2/payments/initiates

        :param dict card:
            A non optional dict, the payment

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_authorize(schema=card)
        self.response = self.post_request(IClientConstants.INITIATE_PAYMENT_ENDPOINT, card, self.token)
        return self.response

    def validate(self, otp: dict):
        """

        POST /api/v2/payments/otp

        :param dict otp:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_validate(schema=otp)
        self.response = self.post_request(IClientConstants.VALIDATE_CARD_PAYMENT_ENDPOINT, otp, self.token)
        return self.response

    def preauth_authorization(self, card_pre_auth: dict):
        """

        POST /api/v2/payments/authorise

        :param dict card_pre_auth:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_preauth(schema=card_pre_auth)
        self.response = self.post_request(IClientConstants.PREAUTH_AUTHORIZE_ENDPOINT, card_pre_auth, self.token)
        return self.response

    def payment_capture(self, payment_capture: dict):
        """

        POST /api/v2/payments/capture

        :param dict payment_capture:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_capture(schema=payment_capture)
        self.response = self.post_request(IClientConstants.PAYMENT_CAPTURE_ENDPOINT, payment_capture, self.token)
        return self.response

    def payment_refund(self, payment_refund: dict):
        """

        POST /api/v2/payments/refund

        :param dict payment_refund:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_refund(schema=payment_refund)
        self.response = self.post_request(IClientConstants.PAYMENT_REFUND_ENDPOINT, payment_refund, self.token)
        return self.response

    def payment_cancel(self, payment_cancel: dict):
        """

        POST /api/v2/payments/cancel

        :param dict payment_cancel:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_cancel(schema=payment_cancel)
        self.response = self.post_request(IClientConstants.PAYMENT_CANCEL_ENDPOINT, payment_cancel, self.token)
        return self.response

    def payment_charge_non3d(self, payment_charge: dict):
        """

        POST /api/v2/payments/charge

        :param dict payment_charge:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_charge_non3d(schema=payment_charge)
        self.response = self.post_request(IClientConstants.PAYMENT_CHARGE_ENDPOINT, payment_charge, self.token)
        return self.response

    def payment_charge_3ds(self, payment_charge: dict):
        """

        POST /api/v2/payments/initiates

        :param dict payment_charge:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_charge_3ds(schema=payment_charge)
        self.response = self.post_request(IClientConstants.INITIATE_PAYMENT_ENDPOINT, payment_charge, self.token)
        return self.response

    def payment_charge_3d(self, payment_charge: dict):
        """

        POST /api/v2/payments/initiates

        :param dict payment_charge:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_payment_charge_3d(schema=payment_charge)
        self.response = self.post_request(IClientConstants.INITIATE_PAYMENT_ENDPOINT, payment_charge, self.token)
        return self.response

    def tokenize(self, payment_charge: dict):
        """

        POST /api/v2/payments/tokenize

        :param dict payment_charge:
            A non optional dict, the payload

        :returns Any self.response

        """
        self.requires_token = True
        CardValidator.is_valid_tokenize(schema=payment_charge)
        self.response = self.post_request(IClientConstants.TOKENIZATION_ENDPOINT, payment_charge, self.token)
        return self.response
